/*******************************************************************************
 * DRV8662EVM_REVD
 * Created: June 10, 2010 
 * Desc: The purpose of this program is to control and provide an input signal 
 *  for the DRV8662 RevD EVM.  The program outputs 4 PWM waveforms (a click, 
 *  a sharp click, a soft click, and an alert) in a loop.  This code should only
 *  be used on a DRV8662EVM RevD. 
 * 
 * I/O
 * P1.1 = GAIN0, P1.2 = GAIN1, P1.3 = EN 
 * P2.6 = GAIN0 FET Control, P2.7 = GAIN1 FET Control
 * P3.2 = PWM+, P3.3 = PWM-
 * 
 * Last Modified: June 24, 2011
 * January 17, 2012 - Added uninstrusive check_test_mode function, GPIO Test
 * 
 ******************************************************************************/

#include <msp430g2553.h>
#include "sine1024.h"
#include "triangle1024.h"

// Globals 
unsigned char g_bin;          // Frequency, F[SMCLK] (32 kHz) / SineNumPoints * bin = f[out]
unsigned char g_run;          // Is Output Operational flag
unsigned char g_test_mode;    // Button press detected flag
unsigned char g_gain;         // Active gain setting 
unsigned char g_diff;         // DRV8662 input mode - SE = 0, Diff = 1
unsigned int  g_tick;         // Waveform Duration
int           g_waveform_ndx; // Duty Cycle
int           g_cycles;       // Number of sinewave cycles 

// Function Prototypes
void StartPiezo(void); 
void StopPiezo(void); 
void check_test_mode(void);
void delay(unsigned int delay_time); 

/****
* Desc: main function 
* Params: none
* Returns: none
*/
int main (void) 
{
  WDTCTL = WDTPW + WDTHOLD;   // stop watchdog timer 
  
  // initialize globals 
  g_waveform_ndx=0;
  g_run=0;
  g_test_mode=0;
  g_diff=1;
  
  BCSCTL1 = CALBC1_8MHZ; 	// set DCO to 8MHz, RSEL to 13
  DCOCTL = CALDCO_8MHZ; 	// set MODx to 3
  
  // Port Setup 
  P1DIR |= 0xCE;    // Set P1 direction (SDA, SCL, P1.1-3 = Out)
  P1OUT = 0x00;     // Set P1 output values
  P2DIR |= 0xC0;    // Set P2 direction (P2.6-7 = Out, P2.5 = In)
  P2OUT = 0xC0;     // Set P2 output values
  P2SEL = 0x00;     // Set P2 to digital I/O mode
  P3DIR |= 0x0C;    // Set P3 direction (P3.2-3 = Out)
  P3OUT = 0x00;     // Set P3 direction
  
  // TRIG Interrupt 
  P2IE |= 0x20;     // Enable P2.5 interrupt  
  P2IES |= 0x20;    // P2.5 detect hi/lo edge
  P2IFG &= ~0x20;   // Clear P2.5 interrupt flag
  
  // TimerA1 Setup
  TA1CCR0 = 0x00FF; // Frequency, TimerA counts up to the value in TA1CCR0, 256 counts
  TA1CCR1 = 0x0080; // Initialize 50% duty cycle on PWM pin 1, count to value in TA1CCR1, then turn on
  TA1CCR2 = 0x0080; // Initialize 50% duty cycle on PWM pin 2, count to value in TA1CCR2, then turn on
  P3SEL = 0xC0;     // Set P3.2 and P3.3 to timers TA1 and TA2 respectively 
  
  // GPIO Test On
  P1DIR |= BIT5;
  P2DIR |= BIT0 + BIT2 + BIT4;
  P3DIR |= BIT0 + BIT4 + BIT5 + BIT7;  
  
  P1OUT |= BIT5;
  P2OUT |= BIT0 + BIT2 + BIT4;
  P3OUT |= BIT0 + BIT4 + BIT5 + BIT7;  
  
  P1SEL &= ~(BIT5);
  P2SEL &= ~(BIT0 + BIT2 + BIT4);
  P3SEL &= ~(BIT0 + BIT4 + BIT5 + BIT7);
  
  
  _EINT();          // Enable Interrupts
  
  while(1)
  {
    __no_operation();
    
    //Play Click Effect
    check_test_mode();
    g_cycles = 10;
    g_tick = 5;
    g_bin = 9;				// 32kHz / 1024 * 9 = 281Hz

    StartPiezo();
    delay(2);
    g_run=1;
    while(g_run==1)
    {
      __no_operation();
    }
    check_test_mode();
    delay(g_tick);
    StopPiezo();
    delay(g_tick);
    delay(1500);
    
    //Play Sharp Click Effect
    check_test_mode();
    g_cycles = 2;
    g_tick = 5;
    g_bin = 10;

    StartPiezo();
    delay(2);
    g_run=1;
    while(g_run==1)
    {
      __no_operation();
    }
    check_test_mode();
    delay(g_tick);
    StopPiezo();
    delay(g_tick);
    
    delay(1500);
    
    //Play Soft Effect
    check_test_mode();
    g_cycles = 4;
    g_tick = 21;
    g_bin = 4;

    StartPiezo();
    delay(2);
    g_run=1;
    while(g_run==1)
    {
      __no_operation();
    }
    check_test_mode();
    delay(g_tick);
    StopPiezo();
    delay(g_tick);
    
    delay(1500);
 
    //Play Alert Effect
    check_test_mode();
    g_cycles = 212;
    g_tick = 5;
    g_bin = 8;

    StartPiezo();
    delay(2);
    g_run=1;
    while(g_run==1)
    {
      __no_operation();
    }
    check_test_mode();
    delay(g_tick);
    StopPiezo();
    delay(g_tick);
    
    delay(1500);
  } // end while
}// end main()

/****
* Desc: Delay timer ~1ms
* Params: delay_time - wait in ms 
* Returns: none
*/ 
void delay(unsigned int delay_time)
{
  unsigned int time_const;
  unsigned int cnt;
  
  for(cnt = 0; cnt < delay_time; cnt++)
  {
     time_const = 1975; 
     do time_const--;
     while (time_const != 0);   
  }
}

/****
* Desc: This function detects an interrupt by checking the g_test_mode flag and 
*   then sets the gain of the amplifier through port P1.1 and P1.2. 
* Params: none
* Returns: none
*/ 
void check_test_mode(void)
{
  
  // Set gain
  switch(g_gain)
  {
  case 0: {P1OUT &= ~(0x06);     // 28 dB, 50 Vpp
            P2OUT &= ~(0xC0); 
            break;}
  case 1: {P1OUT |= 0x02;     // 34 dB, 100 Vpp
            P2OUT |= 0x40; 
            break;} 
  case 2: {P1OUT |= 0x04;     // 38 dB, 150 Vpp
            P2OUT |= 0x80; 
            break;} 
  case 3: {P1OUT |= 0x06;     // 40 dB, 200 Vpp
            P2OUT |= 0xC0;       
            break;}
  }
  
  if(g_test_mode == 1)
  {
    LPM4;     // enter low power mode      
  }
}

/****
* Desc: Begin Piezo PWM output on pins P3.2 (PWM+) and P3.3 (PWM-)
* Params: none
* Returns: none
*/
void StartPiezo(void)
{
    if(g_diff == 1)
    {
      P3SEL = 0x0C;                 // Enable differential output 
    }
    else
    {
      P3SEL = 0x04;                 // Enable single-ended output
    }  
    
    TAR = 0;                        // Reset PWM Count
    P1OUT |= 0x08;                  // Enable Amplifer
    TA1CCTL1 |= OUTMOD_7 + CCIE;    // PWM Reset/Set Mode
    TA1CCTL2 |= OUTMOD_3;           // PWM Set/Reset Mode
    TA1CTL = 0x0210;                // Set TimerA Up mode, SMCLK as TACLK
    
} // end of StartPiezo() 

/****
* Desc: Stop Piezo PWM output 
* Params: none
* Returns: none
*/
void StopPiezo(void)
{
    P1OUT &= 0xF7;                  // Disable Amplifer
    TA1CCR1 = 0x80;                 // Set 50% duty cycle
    TA1CCR2 = 0x80;                 // Set 50% duty cycle
    TA1CTL = 0x0000;                // Stop PWM
    TA1CCTL1 &= ~(OUTMOD_7 | OUT);  // PWM output off, output low
    TA1CCTL2 &= ~(OUTMOD_3 | OUT);  // PWM output off, output low
    g_waveform_ndx = 0;
    
} // end of StopPiezo


/****
* Desc: Interrupt Routine for Piezo PWM output 
* Params: none
* Returns: none 
*/ 
#pragma vector=TIMER1_A1_VECTOR
__interrupt void Timer1_A1_ISR (void)
{
  if(g_run) // If PWM is enabled 
  {
    TA1CCR1 = sinewaveform[g_waveform_ndx];   // Set duty cycle for PWM1, TACCR1 / TACCR0
    TA1CCR2 = sinewaveform[g_waveform_ndx];   // Set duty cycle for PWM2, TACCR2 / TACCR0
    
    g_waveform_ndx = g_waveform_ndx + g_bin;  // Skip points to obtain appropriate frequency  
    
    if(g_waveform_ndx >= NUM_PTS-1)           // If current cycle is over 
    {
      g_cycles--;                             // decrement number of cycles 
    }    
    
    if(g_cycles == 0)   // If waveform complete, cycles = 0
    {
      g_run=0;          // Disable output                              
    }
    
    g_waveform_ndx &= 0x03FF;    // Implement Accumulator (x3ff = Modulo 1024)
  }
  TA1CCTL1 &= ~CCIFG;   // Clear Interrupt Flag
  
} // end of Piezo PWM interrupt routine


/*****
* Desc: Port 2 Interrupt Service Routine for TRIG button
* Params: none
* Returns: none
*/ 
#pragma vector=PORT2_VECTOR
__interrupt void Port_2(void)
{
  __bis_SR_register(LPM4_EXIT);
  g_test_mode ^= 0x01;            // Toggle TRIG interrupt flag
  P2IFG &= ~0x20;                 // Clear P2.5 interrupt flag
  
  if(g_test_mode==1)
  {
    //P1REN = 0x06;
    P1OUT |= 0x08;
  }
  else
  {
    if(g_gain == 3)                 // If set to highest gain
    {
      g_gain = 0;                   // Reset to lowest gain
    }
    else
    {
      g_gain = g_gain + 1;          // Increment gain
    }
    //P1REN = 0x00;
    //P1OUT &= 0x06;
  }
 }